;===============================================================================
;
; Andrew Jacobs
; 2018-10-11
;-------------------------------------------------------------------------------

		.65816
		
TRACE		.equ	0		; Set to 1 to print PH/PL during calc.

;===============================================================================
; Workspace
;-------------------------------------------------------------------------------

		.page0
		.org	$10
		
PL		.space	2		; Result
PH		.space	2
MU		.space	2		; Multiplicand

;===============================================================================
; The emulator expects the S28 to contain a ROM image so this is kicked off via
; the dummy reset vector.
;-------------------------------------------------------------------------------

		.code
		.org	$f000
		
Reset:
		clc			; Switch to native mode
		xce
		rep	#$30		; A/X/Y 16-bit
		.longa	on
		.longi	on
		
; Try combinations of signs

		lda	#4		; Load multiplier
		ldx	#3		; .. and multiplicand
		jsr	BoothMult
		jsr	Hex8
		jsr	NewLine
		
		lda	#-4		; Load multiplier
		ldx	#3		; .. and multiplicand
		jsr	BoothMult
		jsr	Hex8
		jsr	NewLine

		lda	#4		; Load multiplier
		ldx	#-3		; .. and multiplicand
		jsr	BoothMult
		jsr	Hex8
		jsr	NewLine

		lda	#-4		; Load multiplier
		ldx	#-3		; .. and multiplicand
		jsr	BoothMult
		jsr	Hex8
		jsr	NewLine
		
		lda	#-32768		; Load multiplier
		ldx	#-32768		; .. and multiplicand
		jsr	BoothMult
		jsr	Hex8
		jsr	NewLine
		

AllDone:
		wdm	#$ff		; Stop the emulator
		bra	AllDone	

; Output Utilities

		.longa	on
		.longi	on
Hex8:
		pha			; Save A
		txa			; Print X
		jsr	Hex4
		pla			; Then  A
Hex4:
		pha			; Save A
		xba			; Show hi byte
		jsr	Hex2
		pla			; Then lo byte
Hex2:
		pha			
		lsr	a		; Show hi nybble
		lsr	a
		lsr	a
		lsr	a
		jsr	Hex
		pla			; Then lo nybble
Hex:
		sep	#$20
		.longa	off
		and	#$0f		; Binary -> ASCII
		sed
		clc
		adc	#$90
		adc	#$40
		cld
		rep	#$20
		.longa	on

UartTx:
		wdm	#$01		; Print a character
		rts

NewLine:
		lda	#10		; LF
		bra	UartTx

;-------------------------------------------------------------------------------

		.longa	on
		.longi	on
BoothMult:
		sta	PL		; P =  multiplier
		stz	PH
		stx	MU		; A =  multiplicand
		
		clc
		jsr	BoothStep	; Perform 16 multiply steps

		lda	PL		; Load the result
		ldx	PH
		rts
		
BoothStep:
		jsr	$+3		; x16
		jsr	$+3		; x8
		jsr	$+3		; x4
		jsr	$+3		; x2
		
		.if	TRACE
		php
		php
		lda	PH
		jsr	Hex4
		lda	PL
		jsr	Hex4
		lda	#'-'
		jsr	UartTx	
		plp
		lda	#'0'
		adc	#0
		jsr	UartTx
		lda	#'-'
		jsr	UartTx
		lda	#'>'
		jsr	UartTx
		plp
		.endif

		lda	PL
		if cc
		 ror	a
		 lda	PH
		 if cs
		  sbc	MU		; 10: P += S (or P -= A)
		  sta	PH
		 endif
		else
		 ror	a
		 lda	PH
		 if cc
		  adc 	MU		; 01: P += A
		  sta	PH
		 endif
		endif
 		rol	a		; ASR P
		ror	PH
		ror	PL
	
		.if	TRACE
		php
		php
		lda	PH
		jsr	Hex4
		lda	PL
		jsr	Hex4
		lda	#'-'
		jsr	UartTx	
		plp
		lda	#'0'
		adc	#0
		jsr	UartTx
		jsr	NewLine
		plp
		.endif
		
		rts
		
		.org	$fffc
		.word	Reset		; Dummy RESET vector
		
		.end		
